@@ -1,19 +1,20 @@
+from __future__ import annotations
+
 import asyncio
 import itertools
 import logging
 import os
 import threading
-import warnings
 import weakref
 from collections import deque, namedtuple
 
 from tornado.concurrent import Future
 from tornado.ioloop import IOLoop
 
-from ..protocol import nested_deserialize
-from ..utils import get_ip
-from .core import Comm, CommClosedError, Connector, Listener
-from .registry import Backend, backends
+from distributed.comm.core import Comm, CommClosedError, Connector, Listener
+from distributed.comm.registry import Backend, backends
+from distributed.protocol import nested_deserialize
+from distributed.utils import get_ip
 
 logger = logging.getLogger(__name__)
 
@@ -30,16 +31,18 @@ class Manager:
     def __init__(self):
         self.listeners = weakref.WeakValueDictionary()
         self.addr_suffixes = itertools.count(1)
-        with warnings.catch_warnings():
-            # Avoid immediate warning for unreachable network
-            # (will still warn for other get_ip() calls when actually used)
-            warnings.simplefilter("ignore")
-            try:
-                self.ip = get_ip()
-            except OSError:
-                self.ip = "127.0.0.1"
+        self._ip = None
         self.lock = threading.Lock()
 
+    @property
+    def ip(self):
+        if not self._ip:
+            try:
+                self._ip = get_ip()
+            except OSError:
+                self._ip = "127.0.0.1"
+        return self._ip
+
     def add_listener(self, addr, listener):
         with self.lock:
             if addr in self.listeners:
@@ -87,6 +90,13 @@ class QueueEmpty(Exception):
     pass
 
 
+def _set_result_unless_cancelled(fut, result):
+    """Helper setting the result only if the future was not cancelled."""
+    if fut.cancelled():
+        return
+    fut.set_result(result)
+
+
 class Queue:
     """
     A single-reader, single-writer, non-threadsafe, peekable queue.
@@ -118,7 +128,7 @@ class Queue:
         if fut is not None:
             assert len(q) == 0
             self._read_future = None
-            fut.set_result(value)
+            _set_result_unless_cancelled(fut, value)
         else:
             q.append(value)
 
@@ -152,13 +162,18 @@ class InProc(Comm):
 
     _initialized = False
 
-    def __init__(
-        self, local_addr, peer_addr, read_q, write_q, write_loop, deserialize=True
+    def __init__(  # type: ignore[no-untyped-def]
+        self,
+        local_addr: str,
+        peer_addr: str,
+        read_q,
+        write_q,
+        write_loop,
+        deserialize: bool = True,
     ):
-        Comm.__init__(self)
+        super().__init__(deserialize=deserialize)
         self._local_addr = local_addr
         self._peer_addr = peer_addr
-        self.deserialize = deserialize
         self._read_q = read_q
         self._write_q = write_q
         self._write_loop = write_loop
@@ -169,20 +184,26 @@ class InProc(Comm):
         self._initialized = True
 
     def _get_finalizer(self):
-        def finalize(write_q=self._write_q, write_loop=self._write_loop, r=repr(self)):
+        r = repr(self)
+
+        def finalize(write_q=self._write_q, write_loop=self._write_loop, r=r):
             logger.warning(f"Closing dangling queue in {r}")
             write_loop.add_callback(write_q.put_nowait, _EOF)
 
         return finalize
 
     @property
-    def local_address(self):
+    def local_address(self) -> str:
         return self._local_addr
 
     @property
-    def peer_address(self):
+    def peer_address(self) -> str:
         return self._peer_addr
 
+    @property
+    def same_host(self) -> bool:
+        return True
+
     async def read(self, deserializers="ignored"):
         if self._closed:
             raise CommClosedError()
@@ -246,6 +267,14 @@ class InProcListener(Listener):
         self.deserialize = deserialize
         self.listen_q = Queue()
 
+    async def _handle_stream(self, comm):
+        try:
+            await self.on_connection(comm)
+        except CommClosedError:
+            logger.debug("Connection closed before handshake completed")
+            return
+        await self.comm_handler(comm)
+
     async def _listen(self):
         while True:
             conn_req = await self.listen_q.get()
@@ -261,12 +290,7 @@ class InProcListener(Listener):
             )
             # Notify connector
             conn_req.c_loop.add_callback(conn_req.conn_event.set)
-            try:
-                await self.on_connection(comm)
-            except CommClosedError:
-                logger.debug("Connection closed before handshake completed")
-                return
-            IOLoop.current().add_callback(self.comm_handler, comm)
+            IOLoop.current().add_callback(self._handle_stream, comm)
 
     def connect_threadsafe(self, conn_req):
         self.loop.add_callback(self.listen_q.put_nowait, conn_req)
