@@ -12,6 +12,10 @@ from itertools import chain
 from types import ModuleType
 from typing import Any
 
+from packaging.requirements import Requirement
+
+BOKEH_REQUIREMENT = Requirement("bokeh>=2.4.2,!=3.0.*")
+
 required_packages = [
     ("dask", lambda p: p.__version__),
     ("distributed", lambda p: p.__version__),
